#ifndef DATAPROTO_DEF_H
#define DATAPROTO_DEF_H

#include <stdint.h>

#define CLUSTER_NAME_LEN    16
#define DATETIME_LEN	    20
#define USERNAME_LEN	    32
#define EXT_ID_LEN	    32
#define UNIQUE_KEY_LEN	    80 /* > MAX_CLIENTID_LEN + TIME + EXT_ID_LEN */

struct CalledNumber {
    unsigned int plan:4;
    unsigned int type:3;
    unsigned int ext:1;
    char number[255];
};

struct CallingNumber {
    unsigned int plan:4;
    unsigned int type:3;
    unsigned int ext:1;
    unsigned int screening:2;
    unsigned int spare:3;
    unsigned int presentation:2;
    unsigned int extt:1;
    char number[255];
};

struct CallSubAddress {
    unsigned int spare:3;
    unsigned int indicator:1;
    unsigned int type:3;
    unsigned int ext:1;
    char address[255];
};

struct DisconnectionCause {
    unsigned int location:4;
    unsigned int spare:1;
    unsigned int coding:2;
    unsigned int ext:1;
    //unsigned int recommendation;
    unsigned int cause:7;
    unsigned int aflag:1;
    //unsigned int diagnostic;
} __attribute__((__packed__));

struct DateTime {
    unsigned short year;  
    char month;
    char day; 
    char hour;
    char min;
    char sec;
    char sign;     
    char GMThour; 
    char GMTmin; 
    char DSThour;
    char DSTmin;
} __attribute__((__packed__));

struct CodecInfo {
    unsigned int codec:6;
    unsigned int bflag:1;
    unsigned int aflag:1;
    unsigned short bitrate;
    unsigned int bandwidth;
} __attribute__((__packed__));

struct Bearer {
    unsigned int type:5;
    unsigned int aflag:2;
    unsigned int aspare:1;
    unsigned int bitrate:5;
    unsigned int bflag:2;
    unsigned int bspare:1;
    unsigned int note:5;
    unsigned int nflag:2;
    unsigned int nspare:1;
} __attribute__((__packed__));

struct SAcntAcSesInfo {
    uint32_t dwVoiceUnderrun;
    uint32_t dwVoiceOverrun;
    uint32_t dwFaxUnderrun;
    uint32_t dwFaxOverrun;
    uint16_t wVoiceDefJitter;
    uint16_t wVoiceMaxJitter;
    uint16_t wVoiceTopJitter;
    uint16_t wVoiceAvgJitter;
    uint16_t wFaxDefJitter;
    uint16_t wFaxMaxJitter;
    uint16_t wFaxTopJitter;
    uint16_t wFaxAvgJitter;
    uint16_t wFaxTxPages;
} __attribute__((__packed__));

struct AcntQuery {
    int incomplete;
    //char table[MAX_CONFVAL_LEN];
    char body[MYSQL_MAX_QLEN];
    char values[MYSQL_MAX_VLEN];

    /* configured collector ID (client tag) */
    char collectorID[MAX_CLIENTID_LEN];
    /* call identifier */
    int callID;
    /* incoming parent call identifier */
    int inParentCallID;
    /* outgoing parent call identifier */
    int outParentCallID;

    /* incoming called number */
    struct CalledNumber inCalledNum;
    /* outgoing called number */
    struct CalledNumber outCalledNum;

    /* incoming calling number */
    struct CallingNumber inCallingNum;
    /* outgoing calling number */
    struct CallingNumber outCallingNum;

    /* redirecting number incoming */
    struct CallingNumber redNumIn;
    /* redirecting number outgoing */
    struct CallingNumber redNumOut;

    /* incoming calling subaddress */
    struct CallSubAddress inCallingSA;
    /* outgoing calling subaddress */
    struct CallSubAddress outCallingSA;

    /* incoming called subaddress */
    struct CallSubAddress inCalledSA;
    /* outgoing called subaddress */
    struct CallSubAddress outCalledSA;

    struct DisconnectionCause dc;

    /* date and time of call start */
    struct DateTime startDT;
    /* date and time of call end */
    struct DateTime endDT;
    /* date and time of connection end */
    struct DateTime discDT;

    /* bearer capabilities */
    struct Bearer bearer;

    /* session informations */
    struct SAcntAcSesInfo session_info;

    /* audio codec properties */
    struct CodecInfo aCodec;
     /* fax codec properties */
    struct CodecInfo fCodec;
      /* data codec properties */
    struct CodecInfo dCodec;
     /* TC audio codec properties */
    struct CodecInfo atcCodec;
     /* TC fax codec properties */
    struct CodecInfo ftcCodec;
      /* TC data codec properties */
    struct CodecInfo dtcCodec;
};

static void accountingInit(struct CfgClient *client, struct AcntQuery *query);
static int extractByteField(unsigned char *field);
static int extractWordField(unsigned char *field);
static int extractDwordField(unsigned char *field);
static int extractStringField(unsigned char *field, int flen, char *string);
static int parseForward(unsigned char **buf, unsigned int bleft, int jump);
static int accountingParse(struct AcntQuery *query, unsigned char *data, unsigned int datalen);
static void accountingQueryPrepare(struct CfgData *config, struct CfgClient *client,
    struct AcntQuery *query);
void accountingManager(struct CfgData *config, struct CfgClient *client, 
    unsigned char *data, unsigned int datalen);
int mysqlPut(struct CfgData *config, char *query);
int queueAdd(struct CfgData *config, struct CfgClient *client, char *query);
void accountingQueryAddBody(struct AcntQuery *q, char *str);
void accountingQueryAddValue(struct AcntQuery *q, char *str);
void accountingQueryAddStr(struct AcntQuery *q, char *str);
void accountingQueryAddField(struct AcntQuery *q, char *str);
void accountingQueryAddInt(struct AcntQuery *q, int i);
int formatDateTime(struct DateTime *dt, char *buf);
void collectorTerminate(struct CfgData *config, struct CfgClient *client);
void hexString(void *packet, char *dest, int size);
#endif
/* 
 * vim: softtabstop=4 shiftwidth=4 
 */


