/* utils.c - miscellaneous functions
 *
 * Author: Elisa Manara <e@entropika.net>
 * Last update: Mon Nov 20 10:46:02 CET 2006
 *
 * This file is part of Abilis Collector for *nix.
 *
 * Abilis Collector is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Abilis Collector is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 *                                   
 * You should have received a copy of the GNU General Public License along                        
 * with Abilis Collector; see the file COPYING.  If not, write to the Free                        
 * Software Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. 
 */  

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#include "define.h"

void hexDump(void *packet, int size);
extern int DebugFlag;

/* Print error messages */
void printError(const char *fmt, ...)
{
    va_list ap;
    char f_time[TIME_BUF_LEN];

    va_start(ap, fmt);
    getHumanTime(f_time);
    fprintf(stderr, "[!!][%s] ", f_time);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
    fflush(stderr);
}

/* Print debugging messages */
void printDebug(const char *fmt, ...)
{
    va_list ap;
    char f_time[TIME_BUF_LEN];

    if(! DebugFlag) return;
    va_start(ap, fmt);

    getHumanTime(f_time);
    fprintf(stdout, "[--][%s] ", f_time);
    vfprintf(stdout, fmt, ap);
    va_end(ap);
    fflush(stdout);
}

/* hexDump() wrapper, check if global DebugFlag is set */
void printDump(void *packet, int size)
{
    if(! DebugFlag) return;
    hexDump(packet, size);
}

/* Dump data with exadecimal values */
void hexDump(void *packet, int size)
{
    unsigned char *byte = packet;
    int count = 0;

    printf("Data len %d, dump follows\n", size);
    printf("\t\t%d bytes dump:\n", size);
    printf("\t\t");
    for (; byte < (unsigned char*) (packet+size); byte++) {
        count++;
        printf("%02x", *byte);
        if (count % 2 == 0) printf(" ");
        if (count % 16 == 0) printf("\n\t\t");
    }
    printf("\n\n");
}

void hexString(void *packet, char *dest, int size)
{
    char hex[4];
    unsigned char *byte = packet;
    dest[0] = '\0';
    int count = 0;
    for (; byte < (unsigned char*) (packet+size); byte++) {
        count++;
        printf("%02x\n", *byte);
        snprintf(hex, 4, "%02x ", *byte);
        strncat(dest, hex, 4);
    }
}

void baseName(char *path, char *basename)
{
    if(strrchr(path, '/') == NULL)
	snprintf(basename, MAX_PATH_LEN, "%s", path);
    else
	snprintf(basename, MAX_PATH_LEN, "%s", (strrchr(path, '/')+1)); 
}

void getHumanTime(char *tbuffer)
{
    time_t now;
    now = time(NULL);
    /* update start time */
    strftime(tbuffer, TIME_BUF_LEN, "%F %T", localtime(&now));
}

/* 
 * vim: softtabstop=4 shiftwidth=4 
 */


